"""
Author: James Londal
Copyright: Copyright 2019, Picopie Ltd
License: GNU AFFERO GENERAL PUBLIC LICENSE
Version: 1.0.0
Maintainer: James Londal
"""

#converts SuperScout file to DataFrame
from pyrugga.match import Match

__version__ = "1.4"



"""
Author: James Londal
Copyright: Copyright 2021, Picopie Ltd
License: GNU AFFERO GENERAL PUBLIC LICENSE
Version: 1.0.0
Maintainer: James Londal
"""

import pyrugga as prg
import pandas as pd


def _find_files(path,extension):
    import os
    files = [f for f in os.listdir(path) if os.path.isfile(os.path.join(path, f))]
    for f in files:
        # checks the name of file ends in .xml
        if '.' + extension in f.lower() :
            yield f
            
def _generate_id(df):
    hometeam = df['hometeam'].apply(lambda x: x[:3])[0].title()
    awayteam = df['awayteam'].apply(lambda x: x[:3])[0].title()

    year = df['fixture_date'].apply(lambda x: x[-2:])[0]
    month = df['fixture_date'].apply(lambda x: x[3:-5])[0]
    day = df['fixture_date'].apply(lambda x: x[:2])[0]

    fixture_id = hometeam+awayteam+"_"+day+month+year
    return fixture_id


def get_Events(FILES_LOC):
    """
    Gets a collections of files and appends multiple match events into a single dataframe
    
    get_Events('../../datalake/Rugby/pacific_nations_cup/')
    """    
    df = pd.DataFrame()

    for fn in _find_files(FILES_LOC,'xml'):
        #loads each file and stores them in an list
        match = prg.Match(FILES_LOC + fn) 

        tmp_df = match.events
        tmp_df['x_fixture_id'] = _generate_id(match.summary)

        #this is how we should have done this 
        #tmp_df['fixture_id'] = match.summary['fixture_id']

        df = pd.concat([df,tmp_df]) 
        
    return df
    
#VINNY# Looks lovely!
    
 
# appends multiple match timelines into a single dataframe
def get_Timelines(FILES_LOC):
    
    df = pd.DataFrame()

    for fn in _find_files(FILES_LOC,'xml'):
        #loads each file and stores them in an list
        match = prg.Match(FILES_LOC + fn) 

        tmp_df = match.timeline
        tmp_df['x_fixture_id'] = _generate_id(match.summary)

        #this is how we should have done this 
        #tmp_df['fixture_id'] = match.summary['fixture_id']

        df = pd.concat([df,tmp_df]) 
        
    return df 
#VINNY# Looks lovely! Question... if someone used this and then wanted to add / update it with newer games, what you think the process would be?    
    
    
# appends multiple player summaries into a single dataframe   
def get_Players(FILES_LOC):
    df = pd.DataFrame()

    for fn in _find_files(FILES_LOC,'xml'):
        #loads each file and stores them in an list
        match = prg.Match(FILES_LOC + fn) 

        tmp_df = match.player_summary(norm='mins').reset_index()
        tmp_df['x_fixture_id'] = _generate_id(match.summary)

        #this is how we should have done this 
        #tmp_df['fixture_id'] = match.summary['fixture_id']

        df = pd.concat([df,tmp_df]) 
        
    return df 
    
#VINNY# Class!!! The x_fixture_ID is ideal for an easier to read version. Opta do have APIs etc but this seems properly accessible. Amazing!    



"""
Author: James Londal
Copyright: Copyright 2021, Picopie Ltd
License: GNU AFFERO GENERAL PUBLIC LICENSE
Version: 1.0.0
Maintainer: James Londal
"""

import sys

if sys.version_info[0] < 3:
    from StringIO import StringIO
else:
    from io import StringIO

import pandas as pd

EVENTS = StringIO("""action,event
1,Carry
2,Tackle
3,Pass
4,Kick
5,Scrum
6,Lineout Throw
7,Penalty Conceded
8,Turnover
9,Try
10,Attacking Qualities
11,Goal Kick
12,Missed Tackle
13,Turnover Won
14,Restart
15,Possession
16,Other
17,Period
18,Collection
19,Team Play
20,Ref Review
21,Card
22,Clock
23,Ruck
24,Maul
26,Sequence
27,Lineout Take
28,Offensive Scrum
29,Defensive Scrum
44,Counter Attack
45,Defensive Exits
46,Attacking 22 Entry
41,Players Leavign Field
40,Defensive Actions
42,Player Enters Field
43,Ruck OOA""")

DESCRIPTIONS = StringIO("""qualifier,qualifier_description
101,One Out Drive
102,Pick And Go
103,Other Carry
104,Kick Return
105,Restart Return
106,Support Carry
107,Tackled Dominant
108,Tackled Neutral
109,Crossed Gainline
110,Neutral
111,Failed Gainline
112,Initial Break
113,Supported Break
115,Penalty Won
116,Defender Beaten
117,Try Scored
118,Other
119,Error
120,Kick
121,Pass
122,Off Load
124,Pen Conceded
125,Back Move
126,Lateral Offload
127,Backward Offload
128,Own Player
129,To Ground
130,To Opposition
131,Slow
132,Normal
133,Fast
134,Review
135,Left Play
136,Right Play
137,Open Play
138,Blind Play
139,Pass Play
140,Kick Run Play
141,Set Move Play
142,Chase Tackle
143,Cover Tackle
144,Line Tackle
145,Guard Tackle
146,Edge Tackle
147,Other Tackle
148,Penalty Kick
149,Low
150,High
151,Offload
152,Kick Pressure
153,Complete
154,Forced In Touch
155,Offload Allowed
156,Turnover Won
157,Pen Conceded
158,Try Saver
159,Sack
160,Passive - To Ground
161,Missed
162,Bumped Off
163,Stepped
164,Outpaced
165,Positional
166,Tackled
167,Clean Break
168,Try Scored
169,Complete Pass
170,Ruck Pass
171,Miss Pass
172,Break Pass
173,Forward Pass
174,Incomplete Pass
175,Intercepted Pass
176,Key Pass
177,Off Target Pass
178,Try Pass
179,Receiver
180,Pass Error
181,Right Pass
182,Left Pass
183,Long Pass
184,Short Pass
185,Kick In Play
186,Kick In Own 22
187,Bomb
188,Chip
189,Cross Pitch
190,Territorial
191,Low
192,Kick In Touch (Bounce)
193,Kick In Touch (Full)
194,Error - Charged Down
195,Error - Out Of Play
196,Error - Terratorial Loss
197,Error - Dead Ball
198,Caught Full
199,Collected Bounce
200,In Goal
201,Own Player - Collected
202,Own Player - Failed
203,Pressure Carried Over
204,Pressure In Touch
205,Pressure Error
206,Try Kick
207,Catch And Drive
208,Catch And Pass
209,Won Penalty Try
210,Penalty Won
211,Off The Top
212,Pressured Kick
213,No Pressure Kick
214,Offence
215,Defence
216,Not Releasing
217,Hands In Ruck
218,Wrong Side
219,Offside
220,Offside At Kick
221,Collapsing Maul
222,Scrum Offence
223,Lineout Offence
224,Off Feet At Ruck
225,Not Rolling Away
226,Foul Play - Foot Contact
227,Foul Play - Mid Air Tackle
228,Foul Play - High Tackle
229,Foul Play - Other
230,Other Offence
231,Obstruction
232,Dissent
233,Full Penalty
235,Free Kick
236,No Action
237,Yellow Card
238,Red Card
239,On Report
240,Warning
241,Penalty Try
242,TJ Intervention - Pen
243,Ref Discussion
244,Video Review
245,Advantage
246,Yellow
247,Red
248,White
249,Attempted Intercept
250,Bad Offload
251,Bad Pass
252,Carried Over
253,Carried In Touch
254,Dropped Ball Unforced
255,Forward Pass
256,Lost Ball Forced
257,Accidental Offside
258,Lost In Ruck Or Maul
259,Reset
260,Other Error
261,Kick Error
262,Failure To Find Touch
263,Grounding Ball
264,Defence
265,Attack
266,Forced
267,Unforced
268,Interception
269,In Goal Touchdown
270,Mark
271,Attacking Catch
272,Attacking Loose Ball
273,Defensive Catch
274,Defensive Loose Ball
275,Restart Catch
276,Jackal
277,Success
278,Fail
279,Pressure
280,No Pressure
281,50m Restart Kick
282,22m Restart Kick
283,Forward Play
284,Restart Retained
285,Restart Opp Error
286,Restart Opp Collection
287,Restart Own Error
288,Restart Long
289,Restart Short
290,Restart High
291,Restart Low
292,Restart Out Of Play
293,Restart Not 10
294,Restart Offside
295,Conversion
296,Drop Goal
297,Penalty Goal
298,Goal Missed
299,Goal Kicked
300,Miss Left
301,Miss Right
302,Miss Short
303,Miss Hit Goal Kick
304,Hit Post
305,22m Restart
306,Lineout
307,50m Restart
308,Turnover Won
309,Scrum
310,Tap Pen
311,Kick Return
312,Free Kick
313,End Drop Goal
314,End Try
315,End Kick Out Of Play
316,End Pen Won
317,End Scrum
318,End Turnover
319,End Pen Con
320,End Other
321,End Of Play
322,Start Clock
323,Stop Clock
324,Injury
325,Video Replay
326,Disciplinary
327,End Of Period
328,Other
329,Foul Play?
330,Handling Err?
331,Line Decision?
332,Techinical Offence?
333,Video Ref Awarded
334,Video Ref Dissallowed
335,Disallowed Try
336,Ruck Offence?
337,Good Decision
338,Poor Referee Signal
339,Scrum Formation
340,LO Offence?
341,Techinal offence?
342,Wrong Mark?
343,Non Playing Personnel
344,Foul Play - Fighting
345,Foul Play - Dangerous Throw
346,Box
347,Touch Kick
348,Start Set Lineout Steal
349,Start Set 22m Restart Retained
350,Start Set 50m Restart Retained
351,End Set Kick In Play
352,Other Review
353,Won Outright
354,Won Try
356,Won Free Kick
357,Won Penalty
358,Lost Outright
359,Lost Pen Con
360,Lost Free Kick
361,Lost Reversed
362,Anti-Clockwise
363,Clockwise
364,Positive
365,Neutral
366,Negative
367,Scrum Half Pass
368,Scrum Half Kick
369,Scrum Half Run
370,No 8 Pick Up
371,Throw Front
372,Throw Middle
373,Throw Back
374,Throw 15m+
375,Throw Quick
376,Won Clean
377,Won Tap
378,Won Penalty
379,Won Free Kick
380,Won Other
381,Lost Not Straight
382,Lost Clean
383,Lost Free Kick
384,Lost Penalty
385,Lost Other
386,Lineout Win Front
387,Lineout Win Middle
388,Lineout Win Back
389,Lineout Win 15m+
390,Lineout Win Quick
391,Lineout Steal Front
392,Lineout Steal Middle
393,Lineout Steal Back
394,Lineout Steal 15m+
395,Lineout Steal Quick
396,Clean Catch
397,Clean Tap
398,Ineffective Tap
399,Error Missed Take
400,Error Dropped Take
401,Won Outright
402,Lost Outright
403,Penalty Won
404,Penalty Conceded
405,Try Scored
406,Penalty Try
407,Start Period
408,End Period
409,Tackled Ineffective
410,No 8 Pass
411,End Turnover (Scrum)
412,Start Set Scrum Steal
413,End Set Kick Error
414,End Set Kick In Goal
415,End Set Own Lineout
416,Restart Kick In Touch
417,Lost Handling Error
418,Lost Not 5m
419,Lost Overthrown
420,Catch and Run
421,Wrong Side At Ruck
422,Wrong Side At Maul
423,Deliberate Knock On
424,Foul Play - Late Tackle
425,Charging Into Ruck
426,Collapsing
427,Standing Up
428,Back Rows Unbound
429,Whipping Scrum
430,No 9 Offside
431,Not Straight
432,Delaying Put In
433,Barging
434,Obstruction
435,Offside
436,Taking Man In Air
437,Tap Back
438,Charge Down
439,Accidental Knock On
440,Referee Obstruction
441,Tackle Assist
442,Offside At Ruck - Att
443,Offside At Ruck - Def
444,Offside In General Play
445,Offside At Restart
446,Maul Obstruction
447,TJ Intervention - YC
448,TJ Intervention - RC
449,Boring In
450,Early Engagement
451,Wrong Side Run
452,Unplayable
453,Delaying Throw
454,Closing Gap
455,Early Lift
456,Wrong Numbers
457,Dissent Advance
458,Illegal Binding
459,Other Scrum Offence
460,Dominant Contact
461,Neutral Contact
462,Ineffective Contact
463,Quick throw from knock on
464,5 second rule
465,Lineout Alternative
466,Quick throw from upfield postion
467,Ineffective
468,Concussion
469,Retained Collection
470,FTSU
471,Passive - Upright
472,Neutral
473,Dominant Tackle Contact
474,Neutral Tackle Contact
475,Ineffective Tackle Contact
560,Tackle Arrival 
653,The player successfully kicks a ’50/22’ 
652,The player successfully kicks a ’22/50’ 
653,22 Entry Points – Try without Conversion 
654,22 Entry Points – Try and Conversion 
655,22 Entry Points – Penalty Try
656,22 Entry Points – Penalty Goal 
657,22 Entry Points – Drop Goal
658,22 Entry Outcome - Turnover 
659,22 Entry Outcome – Try 
660,22 Entry Outcome – Scrum Won
661,22 Entry Outcome – Penalty Won 
662,22 Entry Outcome – Penalty Goal Attempt 
663,22 Entry Outcome – Penalty Conceded 
664,22 Entry Outcome – Lineout Won
665,22 Entry Outcome – Kick Turnover 
638,22 Entry Outcome – Drop Goal
637,Captains Referral - Inconclusive 
636,Captains Referral – Overturned 
635,Captains Referral – Upheld
633,Defensive OOA 
632,Attacking OOA 
631,General Catch 
632,Kicked out of 22 
633,Carried out of 22 
628,Failed Exit From 22 
629,Turnover 
630,Try Scored
631,Scrum Won 
632,Penalty  Won 
633,Penalty Conceded
634,Lineout 621 Won 
635,Kicked Out 
636,Kick To Opposition
618,Drop Goal 
619,OOA Entry 15th Entry- Def 
620,OOA Entry 14th Entry- Def 
621,OOA Entry 13th Entry- Def 
622,OOA Entry 12th Entry- Def 
623,OOA Entry 11th Entry- Def 
624,OOA Entry 10th Entry- Def 
625,OOA Entry 9th Entry- Def 
626,OOA Entry 8th Entry- Def 
627,OOA Entry 7th Entry- Def 
628,OOA Entry 6th Entry- Def 
629,OOA Entry 5th Entry- Def 
630,OOA Entry 4th Entry- Def 
631,OOA Entry 3rd Entry- Def 
632,OOA Entry 2nd Entry- Def 
633,OOA Entry 1st Entry- Def 
634,OOA Entry 15th Entry- Att 
635,OOA Entry 14th Entry- Att 
636,OOA Entry 13th Entry- Att 
637,OOA Entry 12th Entry- Att 
638,OOA Entry 11th Entry- Att 
639,OOA Entry 10th Entry- Att 
640,OOA Entry 9th Entry- Att 
641,OOA Entry 8th Entry- Att 
642,OOA Entry 7th Entry- Att 
643,OOA Entry 6th Entry- Att 
644,OOA Entry 5th Entry- Att 
645,OOA Entry 4th Entry- Att 
646,OOA Entry 3rd Entry- Att 
647,OOA Entry 2nd Entry- Att 
648,OOA Entry 1st Entry- Att 
649,Own Team 5th+ Entry 
650,Own Team 4th Entry 
651,Own Team 3rd Entry 
652,Own Team 2nd Entry 
653,Own Team 1st Entry 
654,Penalty Conceded - Def 
655,Penalty Conceded – Att
656,Penalty Won 
657,Turnover Won 
658,Got Cleaned Out 
659,Not Clearing
660,Nuisance 
661,Attended 
662,Secured 
663,Failed Clearout 
664,Cleaned Out 
665,Red Card 
666,Yellow Card 
667,Reversal Front Row Replacement 
668,Reversal Concussion Replacement
669,Reversal Blood Replacement 
566,Front Row Replacement
565,Concussion 
564,Blood Replacement 
563,Injury
562,Tactical
561,Aerial Kick Contest 
562,N/A Ruck Speed 
563,6+ Seconds
564,5-6 Seconds
565,4-5 Seconds
566,3-4 Seconds
567,2-3 Seconds
568,1-2 Seconds
569,0-1 Seconds
570,N/A Gainline
571,Euqal to Gainline
572,Behind Gainline
548,Over Gainline
549,Stamping 
550,Persistent Infringement 
551,Fighting
552,Dissent
553,Professional Foul 
542,Foul Play
543,Charged Down 
544,Dropped Short 
545,Hit Crossbar 
546,Hit Right Post 
547,Hit Left Post 
548,Missed Right 
533,Missed Left 
534,Intercepted Break 
535,Kick Line Break 
536,Line Break
537,Snake 
538,Decoy 
539,Break Assist 
526,Try Assist 
527,Reload 
528,Idle 
505,Stayed on Ground 
506,4+ Tacklers Committed 
507,3 Tacklers Committed 
508,2 Tacklers Committed 
509,1 Tackler Committed
500,0 Tacklers Committed""")

events = pd.read_csv(EVENTS, sep=",")
descriptions = pd.read_csv(DESCRIPTIONS, sep=",")


"""
Author: James Londal
Copyright: Copyright 2019, Picopie Ltd
License: GNU AFFERO GENERAL PUBLIC LICENSE
Version: 1.0.0
Maintainer: James Londal
"""

#converts SuperScout file to DataFrame
from pyrugga.parse_xml import to_df

import sqlite3
from sqlalchemy import create_engine
import pandas as pd
import numpy as np
import os
import uuid
import matplotlib.pyplot as plt
import seaborn as sns
import pkgutil
import sys

class Match:

    events = None
    summary = None
    timeline = None
    players = None

    hometeam = None
    awayteam = None


    def __init__(self, fn_name, zones=None):
        """
        Creates a Match object from a Superscout XML file
        """
        (self.events,self.summary,self.players) = to_df(fn_name)
        self._genTimeLine()

        """
        The default method for defining an region on the pitch.
        This function can be replaced using when calling the __init__
        """
        def _Zones( x ): return round( x / 10)

        if  zones == None:
            self._Zones = _Zones
        else :
            self._Zones = zones

        self.hometeam = self.summary['hometeam'].values[0]
        self.awayteam = self.summary['awayteam'].values[0]

        self.homewin = (self.summary['home_score'] > self.summary['away_score'])[0]

        self.ref_name = self.summary['ref_name'].values[0]

        self.draw = (self.summary['home_score'] == self.summary['away_score'])[0]

    def _genTimeLine(self):
        """
        generates a timeline of match
        """

        tmp_filename = str(uuid.uuid4())
        conn = sqlite3.connect(tmp_filename)

        engine = create_engine('sqlite:///%s' % (tmp_filename))

        self.summary.to_sql('match_summary',engine,if_exists='replace',index=False)
        self.events.to_sql('match_events',engine,if_exists='replace',index=False)

        d = os.path.dirname(sys.modules['pyrugga'].__file__)
        sql = open(os.path.join(d, 'timeline.sql'), 'r').read()

        os.remove(tmp_filename)

        timeline = pd.io.sql.read_sql(sql,conn)

        tmp = self.events.groupby(['team_name','set_num'])['points'].sum().reset_index()
        tmp = pd.pivot_table(tmp, values='points', index=['set_num'],columns=['team_name'], aggfunc=np.sum).cumsum()

        timeline = pd.merge(timeline,tmp,how='left',left_on='set_num',right_on='set_num').interpolate(method='linear')

        timeline['dist_traveled'] = np.sqrt(timeline['dist_traveled'])

        timeline[timeline.columns[-2:-1][0] + "_points"] = np.array((timeline.iloc[:,-2:-1] - timeline.iloc[:,-2:-1].shift(1)).fillna(0)).ravel()
        timeline[timeline.columns[-2:-1][0] + "_points"] = np.array((timeline.iloc[:,-2:-1] - timeline.iloc[:,-2:-1].shift(1)).fillna(0)).ravel()
        self.timeline = timeline


    def getTerritoryX(self,perc=False, event=None, event_type=None, description=None):
        """
        Gets the activities by zone length
        """

        tmp = self.events

        if event is not None :
            tmp = self.events.query('event == "%s"' % (event))

        if event_type is not None :
            tmp = self.events.query('event_type == "%s"' % (event_type))

        if description is not None :
            tmp = self.events.query('description == "%s"' % (description))

        pos = tmp.groupby(['team_name','x_coord']).count()['action_id'].reset_index()

        pos['x_coord'] = pos['x_coord'].apply(self._Zones)
        df2 = pd.pivot_table(pos, values='action_id', index=['x_coord'],columns=['team_name'], aggfunc=np.sum, fill_value=0)

        if perc == True:
            return df2.query('x_coord > 0 and x_coord < 11' )/df2.query('x_coord > 0 and x_coord < 11' ).sum()

        return df2.query('x_coord > 0 and x_coord < 11')


    def getTerritoryY(self,perc=False, event=None, event_type=None, description=None):
        """
        Gets the activities by zone breadth
        """
        tmp = self.events

        if event is not None :
            tmp = self.events.query('event == "%s"' % (event))

        if event_type is not None :
            tmp = self.events.query('event_type == "%s"' % (event_type))

        if description is not None :
            tmp = self.events.query('description == "%s"' % (description))

        pos = tmp.groupby(['team_name','y_coord']).count()['action_id'].reset_index()

        pos['y_coord'] = pos['y_coord'].apply(self._Zones)
        df2 = pd.pivot_table(pos, values='action_id', index=['y_coord'],columns=['team_name'], aggfunc=np.sum, fill_value=0)

        if perc == True:
            return df2.query('y_coord > 0 and y_coord < 11' )/df2.query('y_coord > 0 and y_coord < 11' ).sum()

        return df2.query('y_coord > 0 and y_coord < 11')


    def getTerritory(self,perc=False, event=None, event_type=None, description=None):
        """
        Gets the activities by zone breadth
        """
        tmp = self.events

        if event is not None :
            tmp = self.events.query('event == "%s"' % (event))

        if event_type is not None :
            tmp = self.events.query('event_type == "%s"' % (event_type))

        if description is not None :
            tmp = self.events.query('description == "%s"' % (description))

        pos = tmp.groupby(['team_name','x_coord','y_coord']).count()['action_id'].reset_index()


        pos['y_coord'] = pos['y_coord'].apply(self._Zones)
        pos['x_coord'] = pos['x_coord'].apply(self._Zones)

        df2 = pd.pivot_table(pos, values='action_id', index=['y_coord','x_coord'],columns=['team_name'], aggfunc=np.sum, fill_value=0)

        if perc == True:
            return df2.query('y_coord > 0 and y_coord < 11 and x_coord > 0 and x_coord < 11' )/df2.query('y_coord > 0 and y_coord < 11 and x_coord > 0 and x_coord < 11' ).sum()

        return df2.query('y_coord > 0 and y_coord < 11 and x_coord > 0 and x_coord < 11')


    def getTerritoryMetric(self, event=None, event_type=None, description=None):
        """
        Returns the territory metric
        """
        return pd.DataFrame((self.getTerritoryX(perc=False, event=event, event_type=event_type, description=description) / self.getTerritoryX(perc=False,event=event, event_type=event_type, description=description).sum().sum()).sum()).reset_index().rename(columns={ 0 : 'territory'})


    def heat_map(self, event=None, event_type=None, description=None):
        """
        Draws a heatmap of where the type of events took place

        inputs

            event=
            event_type=
            description=

        outputs

         two heat maps one for each team

        """
        fig = plt.figure(figsize=(20,6))


        fig.suptitle('%s vs. %s ' % (self.hometeam, self.awayteam),fontsize="x-large")

        ax  = plt.subplot(1, 2, 2)
        plt.title(self.hometeam)

        hmap = self.getTerritory(perc=False,event=event, event_type=event_type, description=description).reset_index()\
        .pivot("y_coord","x_coord",self.hometeam).fillna(0).astype(int)

        sns.heatmap(hmap, annot=True, fmt="d", linewidths=.5,ax=ax,cmap="Greens")

        ax  = plt.subplot(1, 2, 1)
        plt.title(self.awayteam)

        hmap = self.getTerritory(perc=False,event=event, event_type=event_type, description=description).reset_index()\
        .pivot("y_coord","x_coord",self.awayteam).fillna(0).astype(int)

        # Draw a heatmap with the numeric values in each cell
        sns.heatmap(hmap, annot=True, fmt="d", linewidths=.5,ax=ax,cmap="Greens")

        plt.show()


    def player_summary(self,norm=None):
        """
        Summary for each player in a team along with option to normalise metrics by
        minutes played, actions involved in or phases
        norm = ['min','actions','phases']
        """

        tmp_filename = str(uuid.uuid4())
        conn = sqlite3.connect(tmp_filename)

        engine = create_engine('sqlite:///%s' % (tmp_filename))

        self.events.to_sql('match_events',engine,if_exists='replace',index=False)
        self.players.to_sql('players',engine,if_exists='replace',index=False)

        d = os.path.dirname(sys.modules['pyrugga'].__file__)
        sql = open(os.path.join(d, 'player_summary.sql'), 'r').read()

        player_summary = pd.io.sql.read_sql(sql,conn)

        os.remove(tmp_filename)

        player_summary['dist_traveled'] = np.sqrt(player_summary['dist_traveled'])

        player_summary = player_summary.set_index(['team_name', 'position', 'players_name'])


        if norm == 'min':
            player_summary.iloc[:,3:] = player_summary.iloc[:,7:].div(player_summary['mins'], axis=0)
        elif norm == 'actions':
            player_summary.iloc[:,3:] = player_summary.iloc[:,7:].div(player_summary['actions'], axis=0)
        elif norm == 'phases':
            player_summary.iloc[:,3:] = player_summary.iloc[:,7:].div(player_summary['phases'], axis=0)

        return player_summary



from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='pyrugga',
      version='1.5.1',
      description='A library to analyse Rugby matches using Opta\'s SuperScout files',
      url='https://github.com/jlondal/pyrugga',
      author='James Londal',
      license='GNU AFFERO GENERAL PUBLIC LICENSE',
      install_requires=[
         'pandas',
         'scipy',
         'scikit-learn',
         'matplotlib',
         'statsmodels',
         'sqlalchemy'
      ],
      packages=['pyrugga'],
      zip_safe=False,
      include_package_data=True
)



"""
Author: James Londal
Copyright: Copyright 2019, Picopie Ltd
License: GNU AFFERO GENERAL PUBLIC LICENSE
Version: 1.0.0
Maintainer: James Londal

Description:

Conversts SuperScout XML file into two dataframes. One containing the events of the match and the other is a summary of the match.
The main function is to_df

"""

import xml.etree.ElementTree as ET
import pandas as pd
import os
from tempfile import NamedTemporaryFile
import tempfile


from pyrugga.lookup import events, descriptions

def get_players_fixture(f_in):
    """
    Extracts a summary of each players actions in the match

    input:
    file location

    

    output:
    -- fixture_code
    -- team_id
    -- player_id
    -- team_name
    -- players_name
    -- min (minutes on pitch)
    -- shirt_no
    -- position

    """
    tree = ET.parse(f_in)
    root = tree.getroot()

    for child in root:
        fixture_code = child.text

    fout = NamedTemporaryFile(mode='wt',delete=False)

    c = 0

    for a in tree.findall(".//Player"):
        out = ''
        if c == 0:
            out = 'fixture_code,'
            for i in a.keys():
                out +=  i + ","

            out = out[:-1] + '\n'

        out += str(fixture_code) + ','
        for i in a.keys():
            out +=  a.get(i) + ","

        out = out[:-1] + '\n'

        c += 1
        fout.write(out)

    fout.close()

    players = pd.read_csv(fout.name)
    os.remove(fout.name)


    players['players_name'] = players['PLFORN'] + ' ' + players['PLSURN']


    players = players.rename(columns={
            'Club' : 'team_id',
            'PLID' : 'player_id',
            'TEAMNAME' : 'team_name' ,
            'MINS' : 'min',
            'Metres' : 'metres' ,
            'PosID' : 'position',
            'ShirtNo' : 'shirt_no'
        })
    players = players[['fixture_code','team_id','player_id','team_name','players_name','min','shirt_no','position']]
    return players


def get_fixture_actions(f_in):
    """
    Returns a dataframe with all the events from a match

    input:
    file location

    output

    -- action_id
    -- action_result
    -- action_type
    -- fixture_code
    -- team_name
    -- match_time
    -- metres
    -- phases
    -- set_num
    -- player_id
    -- ps_id
    """
    tree = ET.parse(f_in)
    root = tree.getroot()

    fout = NamedTemporaryFile(mode='wt',delete=False)

    c = 0

    for a in tree.findall(".//ActionRow"):
        out = ''
        if c == 0:
            for i in a.keys():
                out +=  i + ","

            out = out[:-1] + '\n'
        for i in a.keys():
            out +=  a.get(i) + ","

        out = out[:-1] + '\n'

        c += 1
        fout.write(out)

    fout.close()

    actions = pd.read_csv(fout.name)
    os.remove(fout.name)


    actions = actions.rename(columns={
        'ID' : 'action_id' ,
        'Actionresult' : 'action_result' ,
        'ActionType' : 'action_type',
        'FXID' : 'fixture_code' ,
        'TEAMNAME' : 'team_name' ,
        'MatchTime' : 'match_time',
        'Metres' : 'metres' ,
        'PlayNum' : 'phases' ,
        'SetNum' : 'set_num',
        'PLID' : 'player_id',
        'psID' : 'ps_id'
        })

    actions['action_type'] = pd.to_numeric(actions.action_type)
    actions['action_result'] = pd.to_numeric(actions.action_result)
    actions['qualifier3'] = pd.to_numeric(actions.qualifier3)
    actions['qualifier4'] = pd.to_numeric(actions.qualifier4)
    actions['qualifier5'] = pd.to_numeric(actions.qualifier5)

    return actions


def get_fixture(f_in):
    """
    Returns a dataframe with summary of a match

    input:
    file location

    output:

    -- fixture_code
    -- ref_id
    -- ref_name
    -- fixture_date
    -- fx_week
    -- awayteam
    -- hometeam

    """

    tree = ET.parse(f_in)
    root = tree.getroot()

    for child in root:
        fixture_code = child.text

    fout = NamedTemporaryFile(mode='wt',delete=False)

    c = 0

    for a in tree.findall(".//Data"):
        out = ''
        if c == 0:
            out = 'fixture_code,'
            for i in a.keys():
                out +=  i + ","

            out = out[:-1] + '\n'

        out += str(fixture_code) + ','
        for i in a.keys():
            out +=  a.get(i) + ","

        out = out[:-1] + '\n'

        c += 1

        fout.write(out)

    fout.close()


    match = pd.read_csv(fout.name)
    os.remove(fout.name)

    match = match.rename(columns={
        'REFID' : 'ref_id' ,
        'REFNAME' : 'ref_name' ,
        'FxWeek' : 'fx_week',
        'FxDate' : 'fixture_date'
        })

    match = match[['fixture_code','ref_id','ref_name','fixture_date','fx_week','awayteam','hometeam']]
    return match


def calc_score(row):
    """
    Calculated the points scored in an event

    input:

    a row from a DataFrame

    output:

    points scored

    """
    score = 0
    if row['event'] == "Try":
        score = 5
        if row['event_type'] == 'Won Penalty Try':
            score += 2

    if row['event_type'] == "Conversion" and row['outcome'] == 'Goal Kicked':
        score = 2

    if row['event_type'] == "Penalty Goal" and row['outcome'] == 'Goal Kicked':
        score = 3

    if row['event_type'] == "Drop Goal" and row['outcome'] == 'Goal Kicked':
        score = 3

    return score



def to_df(super_scout_file_name):
    """
    converts a superscout file into three dataframes returned as a list

    input:

    location of a file

    output:

    - match_events
         action_id
         additional
         advantage
         description
         event
         event_type
         fixture_code
         home_team_advantage
         match_time
         metres
         outcome
         period
         phases
         players_name
         position
         ps_endstamp
         ps_timestamp
         set_num
         shirt_no
         team_name
         x_coord
         x_coord_end
         y_coord
         y_coord_end
         points
    - match
        fixture_code
        ref_id
        ref_name
        fixture_date
        fx_week
        awayteam
        hometeam
        home_score
        away_score
    - players
        fixture_code
        team_id
        player_id
        team_name
        players_name
        min (minutes on pitch)
        shirt_no
        position
    """

    actions = get_fixture_actions(super_scout_file_name)
    players = get_players_fixture(super_scout_file_name)
    match = get_fixture(super_scout_file_name)

    actions_players = pd.merge(actions, players,  left_on = 'player_id', right_on = 'player_id')
    actions_players = pd.merge(actions_players, events,  left_on = 'action', right_on = 'action')
    actions_players = pd.merge(actions_players, descriptions, how='left', left_on = 'action_type', right_on = 'qualifier')
    actions_players = pd.merge(actions_players, descriptions, how='left', left_on = 'action_result', right_on = 'qualifier')
    actions_players = pd.merge(actions_players, descriptions, how='left', left_on = 'qualifier3', right_on = 'qualifier')
    actions_players = pd.merge(actions_players, descriptions, how='left', left_on = 'qualifier4', right_on = 'qualifier')
    actions_players = pd.merge(actions_players, descriptions, how='left', left_on = 'qualifier5', right_on = 'qualifier')


    actions_players = actions_players[[
       'fixture_code_x',
        'set_num',
       'action_id' ,
       'match_time',
       'ps_timestamp',
       'ps_endstamp',
       'advantage',
       'team_name',
       'players_name',
        'shirt_no',
        'position',
       'event',
       'qualifier_description_x',
       'qualifier_description_y',
       'x_coord',
       'y_coord',
       'x_coord_end',
       'y_coord_end',
        'metres',
        'phases' ,
        'period',
        'score_advantage'
    ]]

    actions_players.columns = [
       'fixture_code_x',
        'set_num',
       'action_id' ,
       'match_time',
       'ps_timestamp',
       'ps_endstamp',
       'advantage',
       'team_name',
       'players_name',
        'shirt_no',
        'position',
       'event',
       'event_type',
       'description',
       'outcome' ,
       'additional',
       'x_coord',
       'y_coord',
       'x_coord_end',
       'y_coord_end',
        'metres',
        'phases',
        'period',
        'home_team_advantage'
    ]

    actions_players.sort_values(by='action_id',inplace=True)

    actions_team = pd.merge(actions, players[['team_id','team_name']].drop_duplicates(), left_on = 'player_id', right_on = 'team_id')
    actions_team = pd.merge(actions_team, events,  left_on = 'action', right_on = 'action')

    actions_team = pd.merge(actions_team, descriptions, how='left', left_on = 'action_type', right_on = 'qualifier')
    actions_team = pd.merge(actions_team, descriptions, how='left', left_on = 'action_result', right_on = 'qualifier')
    actions_team = pd.merge(actions_team, descriptions, how='left', left_on = 'qualifier3', right_on = 'qualifier')
    actions_team = pd.merge(actions_team, descriptions, how='left', left_on = 'qualifier4', right_on = 'qualifier')
    actions_team = pd.merge(actions_team, descriptions, how='left', left_on = 'qualifier5', right_on = 'qualifier')

    actions_team = actions_team[[
       'fixture_code',
        'set_num',
       'action_id' ,
       'match_time',
       'ps_timestamp',
       'ps_endstamp',
       'advantage',
       'team_name',
       'event',
       'qualifier_description_x',
       'qualifier_description_y',
       'x_coord',
       'y_coord',
       'x_coord_end',
       'y_coord_end',
        'metres',
        'phases',
        'period',
        'score_advantage'

    ]]

    actions_team.columns = [
       'fixture_code',
       'set_num',
       'action_id' ,
       'match_time',
       'ps_timestamp',
       'ps_endstamp',
       'advantage',
       'team_name',
       'event',
       'event_type',
       'description',
       'outcome' ,
       'additional',
       'x_coord',
       'y_coord',
       'x_coord_end',
       'y_coord_end',
        'metres',
        'phases',
        'period',
        'home_team_advantage'
    ]

    actions_team.sort_values(by='action_id',inplace=True)

    match_events = pd.concat([actions_players,actions_team],sort=True).sort_values('action_id')

    del match_events['fixture_code_x']

    match_events['fixture_code'] = min(match_events.fixture_code)
    match_events['position'] = match_events['position'].fillna(0)

    match_events['shirt_no'] = match_events['shirt_no'].fillna(0)

    match_events['points'] = match_events.apply(calc_score, axis=1)

    match = pd.merge(match,match_events.groupby('team_name')\
            ['points'].sum().reset_index(),left_on='hometeam',right_on='team_name').\
            rename(columns={'points':'home_score'})

    match = pd.merge(match,match_events.groupby('team_name')\
            ['points'].sum().reset_index(),left_on='awayteam',right_on='team_name').\
            rename(columns={'points':'away_score'})

    del match['team_name_y']
    del match['team_name_x']

    return (match_events,match,players)
